/*******************************************************************************
 * Copyright  2010 Nokia Corporation. All rights reserved. 
 * Nokia and Nokia Connecting People are registered trademarks of Nokia Corporation.
 * Java and all Java-based marks are trademarks or registered trademarks of Sun Microsystems, Inc.
 * Other product and company names mentioned herein may be trademarks or trade names of their respective owners.
 *
 * Subject to the conditions below, you may, without charge:
 *
 *   Use, copy, modify and/or merge copies of this software and associated documentation files (the Software)
 *
 *   Publish, distribute, sub-license and/or sell new software derived from or incorporating the Software.
 *
 *
 *
 * This license, unmodified, shall be included with all copies or
 * substantial portions of the Software that are distributed in source code form.
 *
 * The Software cannot constitute the primary value of any new software derived from or incorporating the Software.
 *
 * Any person dealing with the Software shall not misrepresent the source of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************/
package com.nokia.example.overlay;
//
// Source File Name:   VideoOverlayMIDlet.java

// Testing MMAPI overlay drawing
// This MIDlet draws ControlBar on Video/Camera
// ControlBar can be used to operate VideoControl

import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;
import javax.microedition.midlet.MIDlet;

public class VideoOverlayMIDlet extends MIDlet implements CommandListener
{
    /** Canvas for video */
    private VideoCanvas videoCanvas;
    /** Canvas for camera viewfinder */
    private VideoCanvas cameraCanvas;
    /** Canvas for displaying snapshot */
    private DisplayCanvas displayCanvas;

    /** Exit command */
    private Command exitCommand = new Command("Exit", Command.EXIT, 1);
    /** Open command to open VideoCanvas */
    private Command openCommand = new Command("Open", Command.SCREEN, 1);

    /** Main menu */
    private static List mainmenuList;

    /** VideoCanvas has camera viewfinder */
    protected static final int CAMERA = 0;
    /** VideoCanvas has video player */
    protected static final int VIDEO = 1;
    /** ControlBar buttons have icons */
    protected static final int ICON = 0;

    public VideoOverlayMIDlet()
    {
        videoCanvas = null;
        cameraCanvas = null;
        displayCanvas = null;
    }

    public void startApp()
    {
        // Creating canvases
        displayCanvas = new DisplayCanvas(this);
        cameraCanvas = new VideoCanvas(this,CAMERA);
        videoCanvas = new VideoCanvas(this,VIDEO);

        // Creating list to select VideoControl type
        mainmenuList  = new List("VideoOverlayMIDlet", Choice.IMPLICIT);
        fillList();

        mainmenuList.addCommand(openCommand);
        mainmenuList.addCommand(exitCommand);
        mainmenuList.setCommandListener(this);
        Display.getDisplay(this).setCurrent(mainmenuList);
    }

    /**
     * fillList()
     * Adds list items to main menu List
     */
    private void fillList()
    {
        mainmenuList.append("Camera", null);
        mainmenuList.append("Video", null);
    }

    public void pauseApp()
    {
    }

    public void destroyApp(boolean unconditional)
    {
        if(Display.getDisplay(this).getCurrent() == cameraCanvas)
            cameraCanvas.stop();
        else if(Display.getDisplay(this).getCurrent() == videoCanvas)
            videoCanvas.stop();
    }

    /**
    * overlayMidletExit()
    * Exiting from the MIDlet
    */
    public void overlayMidletExit()
    {
        destroyApp(false);
        notifyDestroyed();
    }

    /**
     * videoCanvasExit()
     * Exiting from the canvas
     * Coming back to list
     */
    void videoCanvasExit()
    {
        Display.getDisplay(this).setCurrent(mainmenuList);
    }

    /**
     * cameraCanvasCaptured()
     * Setting captured image on DisplayCanvas
     * @param byte pngData[] - imagedata as byte array
     */
    void cameraCanvasCaptured(byte pngData[])
    {
        cameraCanvas.stop();
        Display.getDisplay(this).setCurrent(displayCanvas);
        displayCanvas.setFullScreenMode(cameraCanvas.isFullScreen());
        displayCanvas.setMode(cameraCanvas.isFullScreen());
        displayCanvas.setImage(pngData);
    }
    /**
     * videoCanvasStart(boolean camera)
     * Setting captured image on DisplayCanvas
     * @param int videotype - type of videoCanvas
     * @param int buttontype - 
     * For now only one button type is defined - icon button
     */
    void videoCanvasStart(int videotype, int buttontype)
    {
        if(videotype == CAMERA)
        {
            Display.getDisplay(this).setCurrent(cameraCanvas);
            cameraCanvas.startCanvas(buttontype);
        }
        else
        {
            Display.getDisplay(this).setCurrent(videoCanvas);
            videoCanvas.startCanvas(buttontype);
        }
    }
    /**
     * displayCanvasBack()
     * Setting VideoCanvas as the current displayable and then starting VideoControl
     * @param boolean camera;
     * true - start canvas with camera;
     * false - start canvas with video
     */
    void displayCanvasBack(boolean camera)
    {
        if(camera)
        {
            Display.getDisplay(this).setCurrent(cameraCanvas);
            cameraCanvas.start();
        }
        else
        {
            Display.getDisplay(this).setCurrent(videoCanvas);
            videoCanvas.start();
        }
    }

    public void commandAction(Command c, Displayable s)
    {
        if(c == exitCommand)
        {
            overlayMidletExit();
        }
        else if((s == mainmenuList &&
                 c == List.SELECT_COMMAND) ||
                c == openCommand)
        {
            int i = mainmenuList.getSelectedIndex();
            if(i == 0)
            {
                videoCanvasStart(CAMERA,ICON);
            }
            else if(i == 1)
            {
                videoCanvasStart(VIDEO,ICON);
            }
        }
    }

}
